@@ -10,7 +10,7 @@
 
 /* 
 ** \file torcontrol.cpp
-** \version $Id: torcontrol.cpp 2977 2008-08-17 01:28:25Z edmanm $
+** \version $Id: torcontrol.cpp 3777 2009-05-17 04:18:13Z edmanm $
 ** \brief Object for interacting with the Tor process and control interface
 */
 
@@ -282,11 +282,7 @@ TorControl::onAuthenticated()
   getInfo("version", _torVersion);
   /* We want to use verbose names in events and GETINFO results. */
   useFeature("VERBOSE_NAMES");
-
-  /* The control socket is connected, so we can stop reading from stdout */
-  if (_torProcess)
-    _torProcess->closeStdout();
-  
+ 
   emit authenticated();
 }
 
@@ -646,7 +642,16 @@ TorControl::setLogEvents(uint filter, QObject *obj, QString *errmsg)
   setEvent(TorEvents::LogNotice, obj, filter & LogEvent::Notice, false);
   setEvent(TorEvents::LogInfo  , obj, filter & LogEvent::Info  , false);
   setEvent(TorEvents::LogDebug , obj, filter & LogEvent::Debug , false);
-  return (isConnected() ? setEvents(errmsg) : true);
+
+  if (isConnected()) {
+    bool rc = setEvents(errmsg);
+    if (rc && _torProcess)
+      /* The control socket is connected and the request for log events from
+       * the control port was successful, so we can stop reading from stdout. */
+      _torProcess->closeStdout();
+    return rc;
+  }
+  return true;
 }
 
 /** Register for the events currently in the event list */
@@ -1026,28 +1031,25 @@ TorControl::closeStream(const StreamId &streamId, QString *errmsg)
 AddressMap
 TorControl::getAddressMap(AddressMap::AddressMapType type, QString *errmsg)
 {
-  ControlCommand cmd("GETINFO");
-  ControlReply reply;
   AddressMap addressMap;
+  QStringList entries;
 
   switch (type) {
     case AddressMap::AddressMapConfig:
-      cmd.addArgument("addr-mappings/config");
+      entries = getInfo("address-mappings/config").toStringList();
       break;
     case AddressMap::AddressMapCache:
-      cmd.addArgument("addr-mappings/cache");
+      entries = getInfo("address-mappings/cache").toStringList();
       break;
     case AddressMap::AddressMapControl:
-      cmd.addArgument("addr-mappings/control");
+      entries = getInfo("address-mappings/control").toStringList();
       break;
     default:
-      cmd.addArgument("addr-mappings/all");
+      entries = getInfo("address-mappings/all").toStringList();
   }
 
-  if (send(cmd, reply, errmsg)) {
-    foreach (QString mapping, reply.getData()) {
-      addressMap.add(mapping);
-    }
+  foreach (QString entry, entries) {
+    addressMap.add(entry);
   }
   return addressMap;
 }
