@@ -10,7 +10,7 @@
 
 /*
 ** \file mainwindow.cpp
-** \version $Id: mainwindow.cpp 3019 2008-08-30 01:28:46Z edmanm $
+** \version $Id: mainwindow.cpp 3742 2009-04-29 20:13:52Z edmanm $
 ** \brief Main (hidden) window. Creates tray menu and child windows
 **
 ** Implements the main window. The main window is a hidden window that serves
@@ -28,8 +28,12 @@
 #include <clientstatusevent.h>
 #include <dangerousversionevent.h>
 #include <vmessagebox.h>
+#include <procutil.h>
+
+#include "procutil.h"
 
 #include "mainwindow.h"
+#include "controlpasswordinputdialog.h"
 
 #define IMG_BWGRAPH        ":/images/16x16/utilities-system-monitor.png"
 #define IMG_CONTROL_PANEL  ":/images/16x16/system-run.png"
@@ -141,7 +145,6 @@ MainWindow::MainWindow()
            this, SLOT(onSubprocessFinished(int, QProcess::ExitStatus)));
   connect(_browserProcess, SIGNAL(startFailed(QString)),
            this, SLOT(onBrowserFailed(QString)));
-  _browserProcess->setEnvironment(QProcess::systemEnvironment() << "TZ=UTC");
 
   /* Create a new HelperProcess object, used to start the IM client */
   _imProcess = new HelperProcess(this);
@@ -165,21 +168,11 @@ MainWindow::MainWindow()
          this, SLOT(upnpError(UPNPControl::UPNPError)));
 #endif
 
-  if (TrayIcon::isTrayIconSupported()) {
-    /* Make the tray icon visible */
-    _trayIcon.show();
-    /* Check if we are supposed to show our main window on startup */
-    ui.chkShowOnStartup->setChecked(settings.showMainWindowAtStart());
-    if (ui.chkShowOnStartup->isChecked())
-      show();
-  } else {
-    /* Don't let people hide the main window, since that's all they have. */
-    ui.chkShowOnStartup->hide();
-    ui.btnHide->hide();
-    setMinimumHeight(height()-ui.btnHide->height());
-    setMaximumHeight(height()-ui.btnHide->height());
-    show();
-  }
+  ui.chkShowOnStartup->setChecked(settings.showMainWindowAtStart());
+  if (ui.chkShowOnStartup->isChecked())
+    show(); 
+  /* Optimistically hope that the tray icon gets added. */
+  _trayIcon.show();
 }
 
 /** Destructor. */
@@ -192,6 +185,23 @@ MainWindow::~MainWindow()
   delete _configDialog;
 }
 
+void
+MainWindow::setVisible(bool visible)
+{
+  if (visible) {
+    /* In Gnome, will hide buttons if Vidalia is run on startup. */
+    if (!TrayIcon::isTrayIconSupported()) {
+      /* Don't let people hide the main window, since that's all they have. */
+      ui.chkShowOnStartup->hide();
+      ui.btnHide->hide();
+      /* Causes window to not appear in Enlightenment. */
+      //setMinimumHeight(height()-ui.btnHide->height());
+      //setMaximumHeight(height()-ui.btnHide->height());
+    }
+  }
+  VidaliaWindow::setVisible(visible);
+}
+
 /** Catches and processes Tor client and general status events. */
 void
 MainWindow::customEvent(QEvent *event)
@@ -232,13 +242,15 @@ void
 MainWindow::running()
 {
   VidaliaSettings settings;
-  if (_torControl->isRunning()) {
-    /* Tor may be already running, but we still need to connect to it. So,
-     * update our status now. */ 
-    updateTorStatus(Starting);
-    /* Tor was already running */
-    started();
-  } else if (settings.runTorAtStart()) {
+ 
+  /* Initialize _useSavedPassword to true. If Tor is already running when
+   * Vidalia starts, then there is no point in generating a random password.
+   * If Tor is not already running, then this will be set according to the
+   * current configuration in the start() method.
+   */
+  _useSavedPassword = true;
+ 
+  if (settings.runTorAtStart()) {
     /* If we're supposed to start Tor when Vidalia starts, then do it now */
     start();
   }
@@ -268,6 +280,27 @@ MainWindow::shutdown()
     _proxyProcess->kill();
   }
 
+  /* Kill the browser and IM client if using the new launcher */
+  VidaliaSettings vidalia_settings;
+
+  if (! vidalia_settings.getBrowserDirectory().isEmpty()) {
+    /* Disconnect the finished signals so that we won't try to exit Vidalia again */
+    QObject::disconnect(_browserProcess, SIGNAL(finished(int, QProcess::ExitStatus)), 0, 0);
+    QObject::disconnect(_imProcess, SIGNAL(finished(int, QProcess::ExitStatus)), 0, 0);
+
+    /* Use QProcess terminate function */
+    if (_browserProcess->state() == QProcess::Running)
+      _browserProcess->terminate();
+
+#if defined(Q_OS_WIN)
+    /* Kill any processes which might have been forked off */
+    win32_end_process_by_filename(vidalia_settings.getBrowserExecutable());
+#endif
+
+    if (_imProcess->state() == QProcess::Running)
+      _imProcess->terminate();    
+  }
+
   /* Disconnect all of the TorControl object's signals */
   QObject::disconnect(_torControl, 0, 0, 0);
 
@@ -449,20 +482,77 @@ MainWindow::createMenuBar()
 #endif
 }
 
+/** Start a web browser when given the directory containing the executable and profile */
+void
+MainWindow::launchBrowserFromDirectory()
+{
+  VidaliaSettings settings;
+
+  QString browserDirectory = settings.getBrowserDirectory();
+  QString browserDirectoryFilename = settings.getBrowserExecutable();
+
+  /* Set TZ=UTC (to stop leaking timezone information) and
+   * MOZ_NO_REMOTE=1 (to allow multiple instances of Firefox */
+  QStringList env = QProcess::systemEnvironment();
+  env << "TZ=UTC";
+  env << "MOZ_NO_REMOTE=1";
+  _browserProcess->setEnvironment(env);
+
+  /* The browser is in <browserDirectory>/App/Firefox/<browserDirectoryFilename> */
+  QString browserExecutable =
+    QDir::toNativeSeparators(browserDirectory + "/App/Firefox/" + browserDirectoryFilename);
+  /* The profile is in <browserDirectory>/Data/profile */
+  QString profileDir =
+    QDir::toNativeSeparators(browserDirectory + "/Data/profile");
+
+  /* Copy the profile directory if it's not already there */
+  QDir browserDirObj = QDir(browserDirectory);
+
+  /* Copy the profile directory if it's not already there */
+  if (!browserDirObj.exists("Data/profile")) {
+    browserDirObj.mkdir("Data/profile");
+    copy_dir(browserDirectory + "/App/DefaultData/profile", browserDirectory + "/Data/profile");
+  }
+
+  /* Copy the plugins directory if it's not already there */
+  if (!browserDirObj.exists("Data/plugins")) {
+    browserDirObj.mkdir("Data/plugins");
+    copy_dir(browserDirectory + "/App/DefaultData/plugins", browserDirectory + "/Data/plugins");
+  }
+
+  /* Build the command line arguments */
+  QStringList commandLine;
+  // Is this better or worse than MOZ_NO_REMOTE?
+  //commandLine << "-no-remote";
+  commandLine << "-profile";
+  commandLine << profileDir;
+
+  /* Launch the browser */
+  _browserProcess->start(browserExecutable, commandLine);
+}
+
 /** Starts the web browser and IM client, if appropriately configured */
 void
 MainWindow::startSubprocesses()
 {
   VidaliaSettings settings;
-  QString executable = settings.getBrowserExecutable();
-  
-  if (!executable.isEmpty())
-    _browserProcess->start(executable, QStringList());
+  QString subprocess;
 
-  executable = settings.getIMExecutable();
+  /* Launch the web browser */
+  if (!(subprocess = settings.getBrowserDirectory()).isEmpty()) {
+    /* The user has set BrowserDirectory; use this */
+    launchBrowserFromDirectory();
+  } else if (!(subprocess = settings.getBrowserExecutable()).isEmpty()) {
+    /* BrowserDirectory is not set, but BrowserExecutable is; use this */
+    _browserProcess->setEnvironment(QProcess::systemEnvironment() << "TZ=UTC");
+    _browserProcess->start(subprocess, QStringList());
+  }
 
-  if (!executable.isEmpty())
-    _imProcess->start(executable, QStringList());
+  /* Launch the IM client */
+  subprocess = settings.getIMExecutable();
+
+  if (!subprocess.isEmpty())
+    _imProcess->start(subprocess, QStringList());
   
 }
 
@@ -476,15 +566,60 @@ MainWindow::onSubprocessFinished(int exitCode, QProcess::ExitStatus exitStatus)
   /* Get path to browser and IM client */
   VidaliaSettings settings;
   QString browserExecutable = settings.getBrowserExecutable();
+  QString browserDirectory = settings.getBrowserDirectory();
   QString imExecutable = settings.getIMExecutable();
 
   /* A subprocess is finished if it successfully exited or was never asked to start */
-  bool browserDone = browserExecutable.isEmpty() || _browserProcess->isDone();
+  bool browserDone = (browserExecutable.isEmpty() && browserDirectory.isEmpty()) || _browserProcess->isDone();
   bool imDone = imExecutable.isEmpty() || _imProcess->isDone();
 
   /* Exit if both subprocesses are finished */
-  if (browserDone && imDone)
-    shutdown();
+  if (browserDone && imDone) {
+    if (browserDirectory.isEmpty()) {
+      /* We are using the standard launcher, exit immediately */
+      shutdown();
+    } else {
+      /* We are using the alternate launcher, wait until the browser has really died */
+      QTimer *browserWatcher = new QTimer(this);
+      connect(browserWatcher, SIGNAL(timeout()), this, SLOT(onCheckForBrowser()));
+      browserWatcher->start(2000);
+    }
+  }
+}
+
+/** Called periodically to check if the browser is running. If it is not,
+ * exit Vidalia cleanly */
+void
+MainWindow::onCheckForBrowser()
+{
+/* This only works on Windows for now */
+#if defined(Q_OS_WIN)
+
+  VidaliaSettings settings;
+  QString browserDirectoryFilename = settings.getBrowserExecutable();
+
+  /* Get list of running processes */
+  QHash<qint64, QString> procList = win32_process_list();
+
+  /* On old versions of Windows win32_process_list() will return
+     an empty list. In this case, just keep Vidalia open */
+  if (procList.isEmpty()) {
+    return;
+  }
+
+  /* Loop over all processes or until we find <browserDirectoryFilename> */
+  QHashIterator<qint64, QString> i(procList);
+  while (i.hasNext()) {
+    i.next();
+    if (i.value().toLower() == browserDirectoryFilename) {
+      /* The browser is still running, so Vidalia should keep running too */
+      return;
+    }
+  }
+
+  /* The browser isn't running, exit Vidalia */
+  shutdown();
+#endif  
 }
 
 /** Called when the web browser failed to start, for example, because the path
@@ -759,11 +894,10 @@ MainWindow::start()
   QStringList args;
 
   updateTorStatus(Starting);
-  
+
   /* Check if Tor is already running separately */
   if (net_test_connect(settings.getControlAddress(),
                        settings.getControlPort())) {
-    _controlPassword = settings.getControlPassword();
     started();
     return;
   }
@@ -775,7 +909,7 @@ MainWindow::start()
       touch_file(torrc, true);
     args << "-f" << torrc;
   }
-  
+
   /* Specify Tor's data directory, if different from the default */
   QString dataDirectory = settings.getDataDirectory();
   if (!dataDirectory.isEmpty())
@@ -789,10 +923,13 @@ MainWindow::start()
   /* Add the control port authentication arguments */
   switch (settings.getAuthenticationMethod()) {
     case TorSettings::PasswordAuth:
-      if (settings.useRandomPassword())
+      if (settings.useRandomPassword()) {
         _controlPassword = TorSettings::randomPassword();
-      else
+        _useSavedPassword = false;
+      } else {
         _controlPassword = settings.getControlPassword();
+        _useSavedPassword = true;
+      }
       args << "HashedControlPassword"
            << TorSettings::hashPassword(_controlPassword)
            << "CookieAuthentication"  << "0";
@@ -805,14 +942,6 @@ MainWindow::start()
       args << "CookieAuthentication"  << "0"
            << "HashedControlPassword" << "";
   }
-  
-  /* Add custom user and group information (if specified) */
-  QString user = settings.getUser();
-  if (!user.isEmpty())
-    args << "User" << user;
-  QString group = settings.getGroup();
-  if (!group.isEmpty())
-    args << "Group" << group;
 
   /* This doesn't get set to false until Tor is actually up and running, so we
    * don't yell at users twice if their Tor doesn't even start, due to the fact
@@ -1077,6 +1206,10 @@ MainWindow::authenticate()
   } else if (authMethod == TorSettings::PasswordAuth) {
     /* Get the control password and send it to Tor */
     vNotice("Authenticating using 'hashed password' authentication.");
+    if (_useSavedPassword) {
+      TorSettings settings;
+      _controlPassword = settings.getControlPassword();
+    }
     return _torControl->authenticate(_controlPassword);
   }
   /* No authentication. Send an empty password. */
@@ -1148,19 +1281,49 @@ MainWindow::authenticationFailed(QString errmsg)
 
   /* Parsing log messages is evil, but we're left with little option */
   if (errmsg.contains("Password did not match")) {
-    /* Bad password, so prompt for a new one. */
-    QString password = QInputDialog::getText(this,
-                         tr("Password Authentication Required"),
-                         tr("Please enter your control password (not the hash):"),
-                         QLineEdit::Password);
-    if (!password.isEmpty()) {
-      /* XXX: We should ask the user if they really want to save the password
-       * they just typed in. */
-      TorSettings settings;
-      settings.setAuthenticationMethod(TorSettings::PasswordAuth);
-      settings.setControlPassword(password);
-      settings.setUseRandomPassword(false);
+    ControlPasswordInputDialog dlg;
+    connect(&dlg, SIGNAL(helpRequested(QString)),
+            this, SLOT(showHelpDialog(QString)));
+
+    qint64 torPid = 0;
+
+#if defined(Q_OS_WIN32)
+    QHash<qint64, QString> procs = process_list();
+    foreach (qint64 pid, procs.keys()) {
+      if (! procs.value(pid).compare("tor.exe", Qt::CaseInsensitive)) {
+        torPid = pid;
+        break;
+      }
+    }
+    dlg.setResetEnabled(torPid > 0);
+#else
+    dlg.setResetEnabled(false);
+#endif
+
+    int ret = dlg.exec();
+    if (ret == QDialogButtonBox::Ok) {
+      if (dlg.isSavePasswordChecked()) {
+        TorSettings settings;
+        settings.setAuthenticationMethod(TorSettings::PasswordAuth);
+        settings.setUseRandomPassword(false);
+        settings.setControlPassword(dlg.password());
+        _useSavedPassword = true;
+      } else {
+        _controlPassword = dlg.password();
+        _useSavedPassword = false;
+      }
       retry = true;
+    } else if (ret == QDialogButtonBox::Reset) {
+      if (! process_kill(torPid)) {
+        VMessageBox::warning(this,
+          tr("Password Reset Failed"),
+          p(tr("Vidalia tried to reset Tor's control password, but was not "
+               "able to restart the Tor software. Please check your Task "
+               "Manager to ensure there are no other Tor processes running.")),
+               VMessageBox::Ok|VMessageBox::Default);
+      } else {
+        retry = true;
+      }
     }
   } else {
     /* Something else went wrong */
